@@ -19,8 +19,8 @@ static struct option long_options[] =
 	{"threads",  required_argument, 0, 'T'},
 	{"indel",  required_argument, 0, 'I'},
 	{"phred",  required_argument, 0, 'P'},
-	{"minmatch",  required_argument, 0, 'm'},
-	{"minmatch2",  required_argument, 0, 'p'},
+	{"minvotes",  required_argument, 0, 'm'},
+	{"minvotes2",  required_argument, 0, 'p'},
 	{"mindist",  required_argument, 0, 'd'},
 	{"maxdist",  required_argument, 0, 'D'},
 	{"order",  required_argument, 0, 'S'},
@@ -92,6 +92,10 @@ void print_usage_core_aligner()
 	SUBREADputs("                    subreads that map in consensus) . If paired-end, this gives");
 	SUBREADputs("                    the consensus threshold for the anchor read. 3 by default");
 	SUBREADputs("");
+	SUBREADputs("  -M <int>          Specify the maximum number of mis-matched bases allowed in");
+	SUBREADputs("                    the alignment. 3 by default. Mis-matches found in soft-");
+	SUBREADputs("                    clipped bases are not counted.");
+	SUBREADputs("");
 	SUBREADputs("  -T <int>          Number of CPU threads used, 1 by default.");
 	SUBREADputs("");
 	SUBREADputs("  -I <int>          Maximum length (in bp) of indels that can be detected. 5 by");
@@ -112,10 +116,6 @@ void print_usage_core_aligner()
 	SUBREADputs("                    the mapping output. Note that read mapping is performed at");
 	SUBREADputs("                    color-space.");
 	SUBREADputs("");
-	SUBREADputs("  -M <int>          Specify the maximum number of mis-matched bases allowed in");
-	SUBREADputs("                    the alignment. 3 by default. Mis-matches found in soft-");
-	SUBREADputs("                    clipped bases are not counted.");
-	SUBREADputs("");
 	SUBREADputs("  --sv              Detect structural variants (eg. long indel, inversion,");
 	SUBREADputs("                    duplication and translocation) and report breakpoints. Refer");
 	SUBREADputs("                    to Users Guide for breakpoint reporting.");
@@ -219,27 +219,41 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				core_version_number("Subread-align");
 				return -1;
 			case 'G':
+				if(!is_valid_digit(optarg, "G"))
+					STANDALONE_exit(-1);
 				global_context->config.DP_penalty_create_gap = atoi(optarg);
 				break;
 			case 'Y':
+				if(!is_valid_digit(optarg, "Y"))
+					STANDALONE_exit(-1);
 				global_context->config.DP_match_score = atoi(optarg);
 				break;
 			case 'E':
+				if(!is_valid_digit(optarg, "E"))
+					STANDALONE_exit(-1);
 				global_context->config.DP_penalty_extend_gap = atoi(optarg);
 				break;
 			case 'X':
+				if(!is_valid_digit(optarg, "X"))
+					STANDALONE_exit(-1);
 				global_context->config.DP_mismatch_penalty = atoi(optarg);
 				break;
 			case '3':
+				if(!is_valid_digit(optarg, "3"))
+					STANDALONE_exit(-1);
 				global_context->config.read_trim_3 = atoi(optarg); 
 				break;
 			case '5':
+				if(!is_valid_digit(optarg, "5"))
+					STANDALONE_exit(-1);
 				global_context->config.read_trim_5 = atoi(optarg); 
 				break;
 			case 'J':
 				global_context->config.show_soft_cliping = 0;
 				break;
 			case 'B':
+				if(!is_valid_digit(optarg, "B"))
+					STANDALONE_exit(-1);
 				global_context->config.multi_best_reads = atoi(optarg); 
 
 				if(global_context->config.multi_best_reads<1)
@@ -274,12 +288,19 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				global_context->config.convert_color_to_base = 1;
 				break;
 			case 'D':
+
+				if(!is_valid_digit(optarg, "D"))
+					STANDALONE_exit(-1);
 				global_context->config.maximum_pair_distance = atoi(optarg);
 				break;
 			case 'd':
+				if(!is_valid_digit(optarg, "d"))
+					STANDALONE_exit(-1);
 				global_context->config.minimum_pair_distance = atoi(optarg);
 				break;
 			case 'n':
+				if(!is_valid_digit(optarg, "n"))
+					STANDALONE_exit(-1);
 				global_context->config.total_subreads = atoi(optarg);
 				//global_context->config.total_subreads = min(31,global_context->config.total_subreads );
 				break;
@@ -287,7 +308,10 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				global_context->config.minimum_subread_for_first_read = atof(optarg);
 				break;
 			case 'T':
+				if(!is_valid_digit_range(optarg, "T", 1, 64))
+					STANDALONE_exit(-1);
 				global_context->config.all_threads = atoi(optarg);
+
 				if(global_context->config.all_threads <1) global_context->config.all_threads = 1;
 				if(global_context->config.all_threads > MAX_THREADS) global_context->config.all_threads = MAX_THREADS;
 
@@ -306,6 +330,8 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				strncpy(global_context->config.output_prefix, optarg, MAX_FILE_NAME_LENGTH-1);
 				break;
 			case 'I':
+				if(!is_valid_digit_range(optarg, "I", 0, 200))
+					STANDALONE_exit(-1);
 				global_context->config.max_indel_length = atoi(optarg);
 
 				if(!is_64_bit_computer) global_context->config.max_indel_length = min(global_context->config.max_indel_length , 16); 
@@ -339,6 +365,8 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				}
 				break;
 			case 'M':
+				if(!is_valid_digit(optarg, "M"))
+					STANDALONE_exit(-1);
 				global_context->config.max_mismatch_exonic_reads = atoi(optarg);
 				break;
 			case 'P':
@@ -348,6 +376,8 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 					global_context->config.phred_score_format = FASTQ_PHRED64;
 				break;
 			case 'p':
+				if(!is_valid_digit(optarg, "p"))
+					STANDALONE_exit(-1);
 				global_context->config.minimum_subread_for_second_read = atoi(optarg);
 				break;
 			case 't':
@@ -425,6 +455,9 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				}
 				else if(strcmp("minMappedLength", long_options[option_index].name)==0) 
 				{
+					if(!is_valid_digit(optarg, "minMappedLength"))
+						STANDALONE_exit(-1);
+
 					global_context->config.min_mapped_fraction = atoi(optarg);
 				}
 				else if(strcmp("accurateFusions", long_options[option_index].name)==0) 
@@ -434,6 +467,8 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 				else if(strcmp("SVdetection", long_options[option_index].name)==0) 
 				{
 					global_context -> config.do_structural_variance_detection = 1;
+					global_context -> config.use_memory_buffer = 1;
+					global_context -> config.reads_per_chunk = 600llu*1024*1024;
 				}
 				else if(strcmp("maxRealignLocations", long_options[option_index].name)==0)
 				{
@@ -464,6 +499,11 @@ int parse_opts_aligner(int argc , char ** argv, global_context_t * global_contex
 		}
 	}
 
+	if(argc > optind){
+		SUBREADprintf("Invalid parameter '%s'\n", argv[optind]);
+		return -1;
+	}
+
 	global_context->config.more_accurate_fusions = global_context->config.more_accurate_fusions && global_context->config.do_fusion_detection;
 	if(global_context->config.more_accurate_fusions)
 	{
