@@ -16,8 +16,8 @@ static struct option long_options[] =
 	{"read2",  required_argument, 0, 'R'},
 	{"output",  required_argument, 0, 'o'},
 	{"subreads",  required_argument, 0, 'n'},
-	{"minmatch",  required_argument, 0, 'm'},
-	{"minmatch2",  required_argument, 0, 'p'},
+	{"minvotes",  required_argument, 0, 'm'},
+	{"minvotes2",  required_argument, 0, 'p'},
 	{"singleSAM",  required_argument, 0, '1'},
 	{"pairedSAM",  required_argument, 0, '2'},
 	{"threads",  required_argument, 0, 'T'},
@@ -60,7 +60,6 @@ static struct option long_options[] =
 	{"maxRealignLocations",  required_argument, 0, 0},
 	{"minVoteCutoff",  required_argument, 0, 0},
 	{"minMappedFraction",  required_argument, 0, 0},
-	{"disableBigMargin",  no_argument, 0, 0},
 	{"complexIndels", no_argument, 0, 0},
 	{0, 0, 0, 0}
 };
@@ -93,6 +92,10 @@ void print_usage_core_subjunc()
 	SUBREADputs("                    subreads that map in consensus) . If paired-end, this gives");
 	SUBREADputs("                    the consensus threshold for the anchor read. 1 by default");
 	SUBREADputs("");
+	SUBREADputs("  -M <int>          Specify the maximum number of mis-matched bases allowed in");
+	SUBREADputs("                    the alignment. 3 by default. Mis-matches found in soft-");
+	SUBREADputs("                    clipped bases are not counted.");
+	SUBREADputs("");
 	SUBREADputs("  -T <int>          Number of CPU threads used, 1 by default.");
 	SUBREADputs("");
 	SUBREADputs("  -I <int>          Maximum length (in bp) of indels that can be detected. 5 by");
@@ -112,10 +115,6 @@ void print_usage_core_subjunc()
 	SUBREADputs("                    the mapping output. Note that read mapping is performed at");
 	SUBREADputs("                    color-space.");
 	SUBREADputs("");
-	SUBREADputs("  -M <int>          Specify the maximum number of mis-matched bases allowed in");
-	SUBREADputs("                    the alignment. 3 by default. Mis-matches found in soft-");
-	SUBREADputs("                    clipped bases are not counted.");
-	SUBREADputs("");
 	SUBREADputs("  --SAMinput        Input reads are in SAM format.");
 	SUBREADputs("");
 	SUBREADputs("  --BAMinput        Input reads are in BAM format.");
@@ -221,21 +220,39 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				core_version_number("Subjunc");
 				return -1;
 			case 'G':
+				if(!is_valid_digit(optarg, "G"))
+					STANDALONE_exit(-1);
+
 				global_context->config.DP_penalty_create_gap = atoi(optarg);
 				break;
 			case 'Y':
+				if(!is_valid_digit(optarg, "Y"))
+					STANDALONE_exit(-1);
+
 				global_context->config.DP_match_score = atoi(optarg);
 				break;
 			case 'E':
+				if(!is_valid_digit(optarg, "E"))
+					STANDALONE_exit(-1);
+
 				global_context->config.DP_penalty_extend_gap = atoi(optarg);
 				break;
 			case 'X':
+				if(!is_valid_digit(optarg, "X"))
+					STANDALONE_exit(-1);
+
 				global_context->config.DP_mismatch_penalty = atoi(optarg);
 				break;
 			case '3':
+				if(!is_valid_digit(optarg, "3"))
+					STANDALONE_exit(-1);
+
 				global_context->config.read_trim_3 = atoi(optarg); 
 				break;
 			case '5':
+				if(!is_valid_digit(optarg, "5"))
+					STANDALONE_exit(-1);
+
 				global_context->config.read_trim_5 = atoi(optarg); 
 				break;
 
@@ -274,27 +291,45 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				global_context->config.convert_color_to_base = 1;
 				break;
 			case 'D':
+				if(!is_valid_digit(optarg, "D"))
+					STANDALONE_exit(-1);
+
 				global_context->config.maximum_pair_distance = atoi(optarg);
 				break;
 			case 'd':
+				if(!is_valid_digit(optarg, "d"))
+					STANDALONE_exit(-1);
+
 				global_context->config.minimum_pair_distance = atoi(optarg);
 				if(global_context->config.minimum_pair_distance <0)
 					global_context->config.restrected_read_order = 0;
 				break;
 			case 'n':
+				if(!is_valid_digit(optarg, "n"))
+					STANDALONE_exit(-1);
+
 				global_context->config.total_subreads = atoi(optarg);
 				//global_context->config.total_subreads = min(31,global_context->config.total_subreads);
 				break;
 			case 'm':
+				if(!is_valid_digit(optarg, "m"))
+					STANDALONE_exit(-1);
+
 				global_context->config.minimum_subread_for_first_read = atoi(optarg);
 				break;
 			case 'T':
+				if(!is_valid_digit_range(optarg, "T", 1, 64))
+					STANDALONE_exit(-1);
+
 				global_context->config.all_threads = atoi(optarg);
 				if(global_context->config.all_threads <1) global_context->config.all_threads = 1;
 				if(global_context->config.all_threads > MAX_THREADS) global_context->config.all_threads = MAX_THREADS;
 
 				break;
 			case 'M':
+				if(!is_valid_digit(optarg, "M"))
+					STANDALONE_exit(-1);
+
 				global_context->config.max_mismatch_exonic_reads = atoi(optarg);
 				global_context->config.max_mismatch_junction_reads = atoi(optarg);
 				break;
@@ -309,6 +344,9 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				strncpy(global_context->config.output_prefix, optarg, MAX_FILE_NAME_LENGTH-1);
 				break;
 			case 'I':
+				if(!is_valid_digit(optarg, "I"))
+					STANDALONE_exit(-1);
+
 				global_context->config.max_indel_length = atoi(optarg);
 
 				if(!is_64_bit_computer) global_context->config.max_indel_length = min(global_context->config.max_indel_length , 16); 
@@ -342,6 +380,9 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 					global_context->config.phred_score_format = FASTQ_PHRED64;
 				break;
 			case 'p':
+				if(!is_valid_digit(optarg, "p"))
+					STANDALONE_exit(-1);
+
 				global_context->config.minimum_subread_for_second_read = atoi(optarg);
 				break;
 			case 'F':
@@ -349,6 +390,9 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				global_context->config.report_sam_file = 0;
 				break;
 			case 'B':
+				if(!is_valid_digit(optarg, "B"))
+					STANDALONE_exit(-1);
+
 				global_context->config.multi_best_reads = atoi(optarg); 
 
 				if(global_context->config.multi_best_reads<1)
@@ -398,6 +442,8 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				}
 				else if(strcmp("junctionIns", long_options[option_index].name)==0)
 				{
+					if(!is_valid_digit(optarg, "junctionIns"))
+						STANDALONE_exit(-1);
 					global_context->config.check_donor_at_junctions=0;
 					global_context->config.limited_tree_scan = 0;
 					global_context->config.max_insertion_at_junctions = atoi(optarg);
@@ -408,6 +454,8 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				}
 				else if(strcmp("minMappedFraction", long_options[option_index].name)==0) 
 				{
+					if(!is_valid_digit(optarg, "minMappedFraction"))
+						STANDALONE_exit(-1);
 					global_context->config.min_mapped_fraction = atoi(optarg);
 				}
 				else if(strcmp("relaxMismatchedBases", long_options[option_index].name)==0) 
@@ -433,6 +481,8 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 				else if(strcmp("SVdetection", long_options[option_index].name)==0) 
 				{
 					global_context -> config.do_structural_variance_detection = 1;
+					global_context -> config.use_memory_buffer = 1;
+					global_context -> config.reads_per_chunk = 300llu*1024*1024;
 				}
 				else if(strcmp("minDistanceBetweenVariants", long_options[option_index].name)==0)
 				{
@@ -488,9 +538,32 @@ int parse_opts_subjunc(int argc , char ** argv, global_context_t * global_contex
 		}
 	}
 
+	if(argc > optind){
+		SUBREADprintf("Invalid parameter '%s'\n", argv[optind]);
+		return -1;
+	}
+
+
 	if(global_context->config.is_SAM_file_input) global_context->config.phred_score_format = FASTQ_PHRED33;
 	global_context->config.more_accurate_fusions = global_context->config.more_accurate_fusions && global_context->config.do_fusion_detection;
 
+	if(global_context->config.more_accurate_fusions)
+	{
+		global_context->config.high_quality_base_threshold = 999999;
+		//#warning "============ REMOVE THE NEXT LINE ======================"
+		global_context->config.show_soft_cliping = 1;
+		//#warning "============ REMOVE ' + 3' FROM NEXT LINE =============="
+		global_context->config.max_mismatch_junction_reads = 0 + 3;
+
+		//#warning "============ REMOVE ' - 1' FROM NEXT LINE =============="
+		global_context->config.do_big_margin_filtering_for_junctions = 1 - 1;
+		global_context->config.total_subreads = 28;
+
+		//#warning "============ REMOVE THE NEXT LINE BEFORE RELEASE ==============="
+		//global_context->config.multi_best_reads = 1;
+	}
+
+
 
 	return 0;
 }
