@@ -371,8 +371,8 @@ typedef struct{
 	short best_second_diff_bases;
 	short realign_flags;
 	short final_quality;
-	short hamming_matched;
-	
+	short chromosomal_length;
+	short MAPQ_adjustment;
 } realignment_result_t;
 
 #define BUCKETED_TABLE_INIT_ITEMS 3
@@ -451,6 +451,7 @@ typedef struct{
 	int output_buffer_item;
 	int output_buffer_pointer;
 	int is_finished;
+	unsigned int all_mapped_reads;
 	subread_lock_t output_lock;
 } thread_context_t;
 
@@ -618,10 +619,11 @@ unsigned short * _global_retrieve_big_margin_ptr(global_context_t * global_conte
 // The first base in the read actually has a larger coordinate than Pos. 
 unsigned int reverse_cigar(unsigned int pos, char * cigar, char * new_cigar);
 
-int chimeric_cigar_parts(global_context_t * global_context , unsigned int sel_pos, int is_first_section_negative_strand, int is_first_section_reversed, char * in_cigar, unsigned int * out_poses, char ** out_cigars, char * out_strands, int read_len, short * out_read_lens);
+int chimeric_cigar_parts(global_context_t * global_context , unsigned int sel_pos, int is_first_section_negative_strand, int is_first_section_reversed, char * in_cigar, unsigned int * out_poses, char ** out_cigars, char * out_strands, int read_len, short * out_read_lens, char * read_name);
 
 void warning_file_limit();
 void quick_sort(void * arr,int arr_size, int compare (void * arr, int l, int r), void exchange(void * arr, int l, int r));
+void basic_sort(void * arr,int arr_size, int compare (void * arr, int l, int r), void exchange(void * arr, int l, int r));
 
 // L_Minus_R should return -1, 0 or 1 when L<R, L==R or L>R.
 // The result is from Small to Large.
@@ -632,4 +634,9 @@ void absoffset_to_posstr(global_context_t * global_context, unsigned int pos, ch
 void test_PE_and_same_chro(global_context_t * global_context , unsigned int pos1, unsigned int pos2, int * is_PE_distance, int * is_same_chromosome , int rlen1, int rlen2);
 
 int FIXLENstrcmp(char * fixed_len, char * rname);
+
+int is_valid_digit(char * optarg, char * optname);
+int is_valid_digit_range(char * optarg, char * optname, int min, int max_inc);
+int is_valid_float(char * optarg, char * optname);
+int exec_cmd(char * cmd, char * outstr, int out_limit);
 #endif
