@@ -14,6 +14,7 @@
 unsigned long long all_counted;
 typedef unsigned int coverage_bin_entry_t;
 int is_BAM_input = 0;
+int max_M = 10;
 char input_file_name[300];
 char output_file_name[300];
 HashTable * cov_bin_table;
@@ -21,6 +22,7 @@ HashTable * cov_bin_table;
 
 static struct option cov_calc_long_options[] =
 {
+	{"maxMOp",required_argument, 0, 'M'},
 	{"primary",no_argument, 0, 0},
 	{0, 0, 0, 0}
 };
@@ -35,7 +37,7 @@ void calcCount_usage()
 	SUBREADputs("");
 	SUBREADputs("Usage");
 	SUBREADputs("");
-	SUBREADputs("  ./coverageCount -i <input_file> -o <output_prefix>");
+	SUBREADputs("  ./coverageCount [options] -i <input_file> -o <output_prefix>");
 	SUBREADputs("");
 	SUBREADputs("Required arguments:");
 	SUBREADputs("");
@@ -44,6 +46,12 @@ void calcCount_usage()
 	SUBREADputs("  -o <string>  Prefix of the output files. Each output file contains Four-byte");
 	SUBREADputs("               integer numbers");
 	SUBREADputs("");
+	SUBREADputs("Optional arguments:");
+	SUBREADputs("");
+	SUBREADputs("  --maxMOp <int> Maximum number of 'M' operations allowed in a CIGAR string.");
+	SUBREADputs("               10 by default. Both 'X' and '=' are treated as 'M' and adjacent");
+	SUBREADputs("               'M' operations are merged in the CIGAR string.");
+	SUBREADputs("");
 }
 
 void add_chro(char *sam_h)
@@ -162,22 +170,22 @@ int covCalc()
 			}
 
 			coverage_bin_entry_t * chrbin = (coverage_bin_entry_t*) bin_entry[0];
-			unsigned int chrlen = (void *)( bin_entry[0]) - NULL;
-			int cigar_sections = RSubread_parse_CIGAR_string(chro, pos, cigar_str, Chros, Staring_Points, Staring_Read_Points, Section_Lengths, &is_junc);
+			unsigned int chrlen = (void *)( bin_entry[1]) - NULL;
+			int cigar_sections = RSubread_parse_CIGAR_string(chro, pos, cigar_str, max_M, Chros, Staring_Points, Staring_Read_Points, Section_Lengths, &is_junc);
 			for(x1 = 0; x1 < cigar_sections; x1++)
 			{
 				unsigned int x2;
-				//printf("%s: %u-%u\n", chro, Staring_Points[x1], Staring_Points[x1]+Section_Lengths[x1]);
 				for(x2 = Staring_Points[x1]; x2<Staring_Points[x1]+Section_Lengths[x1]; x2++)
 				{
-					if(pos+x2 < chrlen)
-					{
+					if(x2 < chrlen) {
 						if(chrbin[x2] <= COVERAGE_MAX_INT)chrbin[x2] ++;
 						all_counted ++;
 						if(all_counted % 10000000 == 0)
-						{
 							SUBREADprintf("Processed %llu bases.\n", all_counted);
-						}
+					} else {
+						SUBREADprintf("%s:%s %u [%s] :: %u-%u\n", line_buffer , chro, pos, cigar_str, Staring_Points[x1], Staring_Points[x1]+Section_Lengths[x1]);
+						SUBREADprintf("Read %s overhangs the boundary of chromosome %s (%u >= %u)\n", line_buffer, chro, x2, chrlen);
+						//exit(-1);
 					}
 				}
 			}
@@ -234,6 +242,7 @@ int cov_calc_main(int argc, char ** argv)
 	int option_index=0;
 	input_file_name[0]=0;
 	output_file_name[0]=0;
+	max_M = 10;
 	is_BAM_input=0;
 	all_counted = 0;
 
@@ -241,9 +250,14 @@ int cov_calc_main(int argc, char ** argv)
 	opterr=1;
 	optopt=63;
 
-	while ((c = getopt_long (argc, argv, "Bi:o:?", cov_calc_long_options, &option_index)) != -1)
+	while ((c = getopt_long (argc, argv, "BM:i:o:?", cov_calc_long_options, &option_index)) != -1)
 		switch(c)
 		{
+			case 'M':
+				if(!is_valid_digit_range(optarg, "maxMOp", 1 , 64))
+					exit(-1);
+				max_M = atoi(optarg);
+			break;
 			case 'i':
 				strcpy(input_file_name, optarg);
 			break;
@@ -265,7 +279,7 @@ int cov_calc_main(int argc, char ** argv)
 		return 0;
 	}
 
-	int is_bam = is_certainly_bam_file(input_file_name, NULL);
+	int is_bam = is_certainly_bam_file(input_file_name, NULL, NULL);
 
 	if(1==is_bam) is_BAM_input = 1;
 	else if(is_bam < 0)
