@@ -36,6 +36,7 @@
 #define GENE_INPUT_SAM_PAIR_2   95
 
 
+#define MIN_FILE_POINTERS_ALLOWED 50
 
 #define FILE_TYPE_SAM     50
 #define FILE_TYPE_BAM     500
@@ -49,6 +50,8 @@
 #define FILE_TYPE_EMPTY   999990
 #define FILE_TYPE_NONEXIST 999999
 
+#define FEATURECOUNTS_BUFFER_SIZE (1024*1024*12)
+
 
 
 #include <stdlib.h>
@@ -88,6 +91,9 @@ typedef struct {
 	unsigned long long input_buff_SBAM_file_start;
 	unsigned long long input_buff_SBAM_file_end;
 
+	unsigned int chunk_number;
+	unsigned int readno_in_chunk;
+
 	unsigned char * input_buff_BIN;
 	int input_buff_BIN_used;
 	int input_buff_BIN_ptr;
@@ -96,8 +102,9 @@ typedef struct {
 	unsigned long long orphant_space;
 	z_stream strm;
 
-	char immediate_last_read_bin[3000];
+	char immediate_last_read_bin[66000];
 	char immediate_last_read_full_name[MAX_READ_NAME_LEN*2 +80 ];
+	int immediate_last_read_flags;
 	int immediate_last_read_bin_len;
 	int immediate_last_read_name_len;
 
@@ -114,16 +121,21 @@ typedef struct {
 	int is_single_end_mode;
 	int force_do_not_sort;
 	int is_finished;
+	int merge_level_finished;
+	int max_file_open_number;
 	subread_lock_t input_fp_lock;
 	subread_lock_t output_header_lock;
+	subread_lock_t unsorted_notification_lock;
 
 	unsigned long long total_input_reads;
 	unsigned long long total_orphan_reads;
 
+	HashTable * unsorted_notification_table;
 	HashTable * sam_contig_number_table;
 	HashTable * bam_margin_table;
 
 	int total_threads;
+	int input_chunk_no;
 	int input_buff_SBAM_size;
 	int input_buff_BIN_size;
 	char tmp_file_prefix[MAX_FILE_NAME_LENGTH];
@@ -132,10 +144,13 @@ typedef struct {
 	int BAM_header_parsed;
 	unsigned int BAM_l_text;
 	unsigned int BAM_n_ref;
+	int is_unsorted_notified;
+	int is_incomplete_BAM;
 
 	void (* reset_output_function) (void * pairer);
 	int (* output_function) (void * pairer, int thread_no, char * rname, char * bin1, char * bin2); 
 	int (* output_header) (void * pairer, int thread_no, int is_text, unsigned int items, char * bin, unsigned int bin_len); 
+	void (* unsorted_notification) (void * pairer, char * bin1, char * bin2); // it is called only once
 	// reserved for the application passing its own data to the output function.
 	void * appendix1;
 	void * appendix2;
@@ -246,7 +261,7 @@ int does_file_exist (char * filename);
 
 double guess_reads_density_format(char * fname, int is_sam, int * min_phred, int * max_phred, int * tested_reads);
 
-FILE * get_temp_file_pointer(char *temp_file_name, HashTable* fp_table);
+FILE * get_temp_file_pointer(char *temp_file_name, HashTable* fp_table, int * close_immediately);
 
 void write_read_block_file(FILE *temp_fp , unsigned int read_number, char *read_name, int flags, char * chro, unsigned int pos, char *cigar, int mapping_quality, char *sequence , char *quality_string, int rl , int is_sequence_needed, char strand, unsigned short read_pos, unsigned short read_len);
 
@@ -265,7 +280,7 @@ void colorread2base(char * read_buffer, int read_len);
 int warning_file_type(char * fname, int expected_type);
 char color2char(char clr, char c1);
 
-int is_certainly_bam_file(char * fname, int * is_firstread_PE);
+int is_certainly_bam_file(char * fname, int * is_firstread_PE, long long  * SAMBAM_header_length);
 
 unsigned long long int sort_SAM_hash(char * str);
 
@@ -277,12 +292,14 @@ int probe_file_type_fast(char * fname);
 void geinput_seek(gene_input_t * input, gene_inputfile_position_t * pos);
 void geinput_tell(gene_input_t * input, gene_inputfile_position_t * pos);
 unsigned long long geinput_file_offset( gene_input_t * input);
+int probe_file_type_EX(char * fname, int * is_first_read_PE, long long * SAMBAM_header_length);
 
 
 int SAM_pairer_create(SAM_pairer_context_t * pairer, int all_threads, int bin_buff_size_per_thread, int BAM_input, int is_Tiny_Mode, int is_single_end_mode, int force_do_not_sort, int display_progress, char * in_file, void (* reset_output_function) (void * pairer), int (* output_header_function) (void * pairer, int thread_no, int is_text, unsigned int items, char * bin, unsigned int bin_len), int (* output_function) (void * pairer, int thread_no, char * rname, char * bin1, char * bin2), char * tmp_path, void * appendix1) ;
 int SAM_pairer_run( SAM_pairer_context_t * pairer);
 void SAM_pairer_destroy(SAM_pairer_context_t * pairer);
 void SAM_pairer_writer_reset(void * pairer);
+void SAM_pairer_set_unsorted_notification(SAM_pairer_context_t * pairer, void (* unsorted_notification) (void * pairer, char * bin1, char * bin2));
 
 int SAM_pairer_multi_thread_output( void * pairer, int thread_no, char * rname, char * bin1, char * bin2 );
 int SAM_pairer_multi_thread_header (void * pairer_vp, int thread_no, int is_text, unsigned int items, char * bin, unsigned int bin_len);
@@ -290,4 +307,6 @@ int SAM_pairer_multi_thread_header (void * pairer_vp, int thread_no, int is_text
 int SAM_pairer_writer_create( SAM_pairer_writer_main_t * bam_main , int all_threads, int has_dummy , int BAM_output, int BAM_compression_level, char * out_file);
 void SAM_pairer_writer_destroy( SAM_pairer_writer_main_t * bam_main ) ;
 int SAM_pairer_iterate_int_tags(unsigned char * bin, int bin_len, char * tag_name, int * saved_value);
+int SAM_pairer_warning_file_open_limit();
+void *delay_realloc(void * old_pntr, size_t old_size, size_t new_size);
 #endif
