@@ -88,6 +88,7 @@ typedef struct
 	unsigned long long input_binary_stream_read_ptr;
 	unsigned long long input_binary_stream_write_ptr;
 	unsigned long long input_binary_stream_buffer_start_ptr;
+	unsigned long long header_length;
 
 	SamBam_Reference_Info * bam_chro_table; 
 	int bam_chro_table_size;
@@ -95,6 +96,7 @@ typedef struct
 
 	char * input_binary_stream_buffer;
 	int is_eof;
+	int is_paired_end;
 } SamBam_FILE;
 
 
@@ -178,6 +180,7 @@ int SamBam_unzip(char * out , char * in , int inlen);
 
 int SamBam_fetch_next_chunk(SamBam_FILE *fp);
 
-int SamBam_compress_cigar(char * cigar, int * cigar_int, int * ret_coverage);
+int SamBam_compress_cigar(char * cigar, int * cigar_int, int * ret_coverage, int max_secs);
 char cigar_op_char(int ch);
+void SamBam_read2bin(char * read_txt, char * read_bin);
 #endif
